# -*- coding: utf-8 -*-
__author__ = 'vahid'

#: Minimum year supported by the library.
MINYEAR = 1

#: Maximum year supported by the library.
MAXYEAR = 3178

#: Representing the Saturday weekday.
SATURDAY = 0

#: Representing the Sunday weekday.
SUNDAY = 1

#: Representing the Monday weekday.
MONDAY = 2

#: Representing the Tuesday weekday.
TUESDAY = 3


#: Representing the Wednesday weekday.
WEDNESDAY = 4


#: Representing the Thursday weekday.
THURSDAY = 5


#: Representing the Friday weekday.
FRIDAY = 6
